'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setProcessLogToGrid = exports.getGridAndScreen = undefined;

var _blessed = require('blessed');

var _blessed2 = _interopRequireDefault(_blessed);

var _blessedContrib = require('blessed-contrib');

var _blessedContrib2 = _interopRequireDefault(_blessedContrib);

var _spawnly = require('spawnly');

var _spawnly2 = _interopRequireDefault(_spawnly);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getGridAndScreen = exports.getGridAndScreen = function getGridAndScreen() {
  var screen = _blessed2.default.screen();
  var grid = new _blessedContrib2.default.grid({ rows: 12, cols: 12, screen: screen }); // eslint-disable-line

  return {
    screen: screen,
    grid: grid
  };
};

var layouts = {
  1: [{ row: 0, col: 0, rowSpan: 12, colSpan: 12 }],
  2: [{ row: 0, col: 0, rowSpan: 12, colSpan: 6 }, { row: 0, col: 6, rowSpan: 12, colSpan: 6 }],
  3: [{ row: 0, col: 0, rowSpan: 6, colSpan: 6 }, { row: 0, col: 6, rowSpan: 6, colSpan: 6 }, { row: 6, col: 0, rowSpan: 6, colSpan: 12 }],
  4: [{ row: 0, col: 0, rowSpan: 6, colSpan: 6 }, { row: 0, col: 6, rowSpan: 6, colSpan: 6 }, { row: 6, col: 0, rowSpan: 6, colSpan: 6 }, { row: 6, col: 6, rowSpan: 6, colSpan: 6 }],
  5: [{ row: 0, col: 0, rowSpan: 6, colSpan: 4 }, { row: 0, col: 4, rowSpan: 6, colSpan: 4 }, { row: 0, col: 8, rowSpan: 6, colSpan: 4 }, { row: 6, col: 0, rowSpan: 6, colSpan: 6 }, { row: 6, col: 6, rowSpan: 6, colSpan: 6 }],
  6: [{ row: 0, col: 0, rowSpan: 6, colSpan: 4 }, { row: 0, col: 4, rowSpan: 6, colSpan: 4 }, { row: 0, col: 8, rowSpan: 6, colSpan: 4 }, { row: 6, col: 0, rowSpan: 6, colSpan: 4 }, { row: 6, col: 4, rowSpan: 6, colSpan: 4 }, { row: 6, col: 8, rowSpan: 6, colSpan: 4 }],
  7: [{ row: 0, col: 0, rowSpan: 4, colSpan: 4 }, { row: 0, col: 4, rowSpan: 4, colSpan: 4 }, { row: 0, col: 8, rowSpan: 4, colSpan: 4 }, { row: 4, col: 0, rowSpan: 4, colSpan: 4 }, { row: 4, col: 4, rowSpan: 4, colSpan: 4 }, { row: 4, col: 8, rowSpan: 4, colSpan: 4 }, { row: 8, col: 0, rowSpan: 4, colSpan: 12 }],
  8: [{ row: 0, col: 0, rowSpan: 4, colSpan: 4 }, { row: 0, col: 4, rowSpan: 4, colSpan: 4 }, { row: 0, col: 8, rowSpan: 4, colSpan: 4 }, { row: 4, col: 0, rowSpan: 4, colSpan: 4 }, { row: 4, col: 4, rowSpan: 4, colSpan: 4 }, { row: 4, col: 8, rowSpan: 4, colSpan: 4 }, { row: 8, col: 0, rowSpan: 4, colSpan: 6 }, { row: 8, col: 6, rowSpan: 4, colSpan: 6 }],
  9: [{ row: 0, col: 0, rowSpan: 4, colSpan: 4 }, { row: 0, col: 4, rowSpan: 4, colSpan: 4 }, { row: 0, col: 8, rowSpan: 4, colSpan: 4 }, { row: 4, col: 0, rowSpan: 4, colSpan: 4 }, { row: 4, col: 4, rowSpan: 4, colSpan: 4 }, { row: 4, col: 8, rowSpan: 4, colSpan: 4 }, { row: 8, col: 0, rowSpan: 4, colSpan: 4 }, { row: 8, col: 4, rowSpan: 4, colSpan: 4 }, { row: 8, col: 8, rowSpan: 4, colSpan: 4 }]
};

// TODO: Generalize this, no need to have fixed layouts.
var getLayoutByIndexAndCount = function getLayoutByIndexAndCount(index, count) {
  var positions = layouts[count];
  return positions[index];
};

var setProcessLogToGrid = exports.setProcessLogToGrid = function setProcessLogToGrid(cmd, grid, index, count) {
  var _getLayoutByIndexAndC = getLayoutByIndexAndCount(index, count),
      row = _getLayoutByIndexAndC.row,
      col = _getLayoutByIndexAndC.col,
      rowSpan = _getLayoutByIndexAndC.rowSpan,
      colSpan = _getLayoutByIndexAndC.colSpan;

  var box = grid.set(row, col, rowSpan, colSpan, _blessed2.default.box, {
    label: cmd.substr(0, 40),
    padding: { top: 1, left: 1, right: 1, bottom: 1 },
    border: {
      type: 'line'
    },
    style: {
      border: {
        fg: 'yellow'
      }
    }
  });

  var log = _blessed2.default.log({
    parent: box,
    tags: true,
    width: '100%-5',
    scrollable: true,
    input: true,
    alwaysScroll: true,
    scrollbar: {
      ch: ' ',
      inverse: true
    },
    keys: true,
    vi: true,
    mouse: true
  });

  var addListener = function addListener(stream) {
    if (!stream.readable) return;
    stream.on('data', function (chunk) {
      return log.log(chunk.toString());
    });
  };

  var cp = void 0;
  return {
    start: function start() {
      cp = (0, _spawnly2.default)(cmd, { stdio: 'pipe', detached: true });

      addListener(cp.stdout);
      addListener(cp.stderr);

      cp.on('close', function (exitCode) {
        log.log('process exit with code ' + exitCode);
        cp.__closed = true;
      });
    },
    stop: function stop() {
      if (cp && !cp.exitCode && !cp.__closed) {
        process.kill(-cp.pid, 'SIGTERM');
      }
    }
  };
};